#include "audio.hpp"


void SoundEngine::ERRCHECK(FMOD_RESULT result)
{
    if (result != FMOD_OK)
    {
        std::cout<<"FMOD error! ("<< result<<") "<< FMOD_ErrorString(result)<<std::endl;
        exit(-1);
    }
}

void SoundEngine::loadsound(const string& s)
{
  sounds.resize(sounds.size()+1);
  channels.resize(sounds.size()+1);

  FMOD_RESULT result;
  result=FMOD_System_CreateSound(fmod_system,s.c_str(),FMOD_HARDWARE|FMOD_2D|FMOD_LOOP_OFF,0, &sounds[sounds.size()-1]);
  ERRCHECK(result);
}

void SoundEngine::play(int i)
{
  FMOD_RESULT result;
	result = FMOD_System_PlaySound(fmod_system, FMOD_CHANNEL_REUSE, sounds[i+1], 0, &channels[i+1]);
  ERRCHECK(result);
}

void SoundEngine::streamtrack(const string& s)
{
	FMOD_RESULT result;

	FMOD_Channel_Stop(channels[0]);

	result = FMOD_System_CreateStream(fmod_system,s.c_str(), FMOD_SOFTWARE | FMOD_2D | FMOD_LOOP_NORMAL, 0, &(sounds[0]));
	ERRCHECK(result);

  result = FMOD_System_PlaySound(fmod_system, FMOD_CHANNEL_REUSE, sounds[0], 0, &channels[0]);
  ERRCHECK(result);
}


void SoundEngine::startdsp()
{
    dsp=0;


    FMOD_RESULT result;
    result = FMOD_System_CreateDSPByType(fmod_system, FMOD_DSP_TYPE_OSCILLATOR, &dsp);
    ERRCHECK(result);
    result = FMOD_DSP_SetParameter(dsp, FMOD_DSP_OSCILLATOR_RATE, 440.0f);       /* musical note 'A' */
    ERRCHECK(result);

    result = FMOD_System_PlayDSP(fmod_system, FMOD_CHANNEL_REUSE, dsp, 1, &dspchannel);
    FMOD_Channel_SetVolume(dspchannel, 0.1f);
    result = FMOD_DSP_SetParameter(dsp, FMOD_DSP_OSCILLATOR_TYPE, 5);
    ERRCHECK(result);
    FMOD_Channel_SetPaused(dspchannel, 0);

}

void SoundEngine::setnoisevol(float v)
{
    curvol=v;
    FMOD_Channel_SetVolume(dspchannel, v);
}


void SoundEngine::amplify()
{
    if (curvol<0.3f)
    {
        curvol+=0.008;
        FMOD_Channel_SetVolume(dspchannel, curvol);
    }
}

void SoundEngine::diminish()
{
    if (curvol>0.1f)
    {
        curvol-=0.008;
        FMOD_Channel_SetVolume(dspchannel, curvol);
    }
}

SoundEngine::~SoundEngine()
{
  FMOD_RESULT result;

  foreach (FMOD_SOUND* s,sounds)
  {
	 result = FMOD_Sound_Release(s);
	 ERRCHECK(result);
  }

  result = FMOD_System_Close(fmod_system);
  ERRCHECK(result);

  result = FMOD_System_Release(fmod_system);
  ERRCHECK(result);
}



SoundEngine::SoundEngine()
{
  FMOD_RESULT       result;
	unsigned int      version;

  result = FMOD_System_Create(&(fmod_system));
  ERRCHECK(result);

  result = FMOD_System_GetVersion(fmod_system, &version);
  ERRCHECK(result);

  if (version < FMOD_VERSION)
  {
		std::cout<<"VERSION ERROR"<<std::endl;
  }

	result = FMOD_System_Init(fmod_system,25, FMOD_INIT_NORMAL, NULL);
  ERRCHECK(result);

  curvol=0.1f;

  sounds.resize(1);
  channels.resize(1);
}
