#include "gestures.hpp"


MouseGesture::MouseGesture()
{
    init();
}

void MouseGesture::addGesture(string gesture,int index)
{
    vector<float> g;
    g.clear();
    for (unsigned int i=0;i<gesture.size();i++)
    {
        g.push_back(gesture[i]- '0');
    }

    gestures.push_back(g);
    indices.push_back(index);
}

void MouseGesture::init()
{

    // Build the angles map
    buildAnglesMap();

    // Gesture Spots
    gestures.clear();
    indices.clear();
    results.clear();

}

void MouseGesture::buildAnglesMap()
{

    // Angle of one sector
    sectorRad=3.1415926535f*2.0f/DEFAULT_NB_SECTORS;

    // map containing sectors no from 0 to PI*2
    anglesMap.clear();

    // the precision is PI*2/100
    float step=3.1415926535f*2/100;

    // memorize sectors
    float sector;
    for (float i=-sectorRad/2;i<=3.1415926535*2-sectorRad/2;i+=step)
    {
        sector=floor((i+sectorRad/2)/sectorRad);
        anglesMap.push_back(sector);
    }
    int a=4;
    a++;
}

void MouseGesture::addMove(int dx,int dy)
{
    float angle=atan2f(dy,dx)+sectorRad/2;
    if (angle<0)angle+=3.1415926535*2;

    int no=(int)(angle/(3.1415926535f*2.0f)*100.0f);
    moves.push_back(anglesMap[no]);
}

void MouseGesture::captureMouse(int msx, int msy)
{
    int difx=msx-lastPoint.x;
    int dify=msy-lastPoint.y;
    float sqDist=difx*difx+dify*dify;
    float sqPrec=DEFAULT_PRECISION*DEFAULT_PRECISION;

    if (sqDist>sqPrec)
    {
        points.push_back((Point)
        {
            msx,msy
        }
                        );
        addMove(difx,dify);
        lastPoint.x=msx;
        lastPoint.y=msy;

    }


}

void MouseGesture::startCapture(int mx, int my)
{

    // moves
    moves.clear();
    points.clear();


    // last point
    lastPoint.x=mx;
    lastPoint.y=my;

}

int MouseGesture::matchGesture()
{

    unsigned int bestCost=1000000;
    unsigned int nbGestures=gestures.size();
    unsigned int cost;
    string gest;
    int bestgesture=-1;

    for (unsigned int i=0;i<nbGestures;i++)
    {

        vector<float> gest=gestures[i];

        //infos.datas=gestures[i].datas;

        cost=costLeven(gest,moves);
        if (cost<=DEFAULT_FIABILITY)
        {
            /*		if (gestures[i].match!=null){
            			infos.cost=cost;
            			cost=gestures[i].match(infos);
            		} <-dunno about this stuff yet */
            if (cost<bestCost)
            {
                bestCost=cost;
                bestgesture=i;
            }
        }
    }


    if (bestgesture>=0)
    {
        if (bestCost<14)
        {
            return indices[bestgesture];
        }
        else
            return -1;
    }
    else
    {
        return -1;
        //no event :'(
    }
}

unsigned int MouseGesture::difAngle(int a,int b)
{
    unsigned int dif=abs(a-b);
    if (dif>DEFAULT_NB_SECTORS/2)
        dif=DEFAULT_NB_SECTORS-dif;
    return dif;
}

void MouseGesture::fill2DTable(unsigned int w, unsigned int h, unsigned int f)
{
    table.resize(boost::extents[w][h]);

    for (unsigned int x=0;x<w;x++)
    {
        for (unsigned int y=0;y<h;y++)
            table[x][y]=f;
    }
}

unsigned int MouseGesture::costLeven(const vector<float>& a, const vector<float>& b)
{

    // point
    if (a[0]==-1 || (b.size()==0))
    {
        return b.size()== 0 ? 0 : 100000;
    }

    // precalc difangles
    fill2DTable(a.size()+1,b.size()+1,0);
    boost::multi_array<unsigned int, 2> w=table;

    for (unsigned int x=1;x<=a.size();x++)
    {
        for (unsigned int y=1;y<b.size();y++)
        {
            table[x][y]=difAngle(a[x-1],b[y-1]);
        }
    }

    // max cost
    for (unsigned int y=1;y<=b.size();y++)
        w[0][y]=100000;
    for (unsigned int x=1;x<=a.size();x++)
        w[x][0]=100000;

    w[0][0]=0;

    // levensthein application
    unsigned int cost=0;
    unsigned int pa;
    unsigned int pb;
    unsigned int pc;

    for (unsigned int x=1;x<=a.size();x++)
    {
        for (unsigned int y=1;y<b.size();y++)
        {
            cost=table[x][y];
            pa=w[x-1][y]+cost;
            pb=w[x][y-1]+cost;
            pc=w[x-1][y-1]+cost;
            w[x][y]=min(min(pa,pb),pc);
        }
    }

    return w[a.size()-1][b.size()-1];
}
