#ifndef GESTURES_HPP
#define GESTURES_HPP

#include <string>
#include <iostream>
#include <vector>
using namespace std;

#include "boost/multi_array.hpp"

/*
 approach based on this one in as3:

 http://drawlogic.com/2007/05/20/as3-mouse-gesture-recognition/

*/



    class Point {
      public:
      float x;
      float y;
    };


    class MouseGesture {
		public:
		static const unsigned int DEFAULT_NB_SECTORS=8;		// Number of sectors
        static const unsigned int DEFAULT_TIME_STEP=20;		// Capture interval in ms
        static const unsigned int DEFAULT_PRECISION=20;		// Precision of catpure in pixels
        static const unsigned int DEFAULT_FIABILITY=30;		// Default fiability level

        vector<vector <int> > results;

        public:
        vector<float> moves;						// Mouse gestures
        vector<Point> points;
        vector<vector<float> > gestures;
        vector<int> indices;
        private:
        Point lastPoint;					// Last mouse point
        unsigned int captureDepth;

        boost::multi_array<unsigned int, 2> table;

		float sectorRad;					// Angle of one sector
		vector<float> anglesMap;					// Angles map


        public:

        MouseGesture();
        void init();


        void addGesture(string gesture,int index);


		// ------------------------------------------------
		//
		// ---o private methods
		//
		// ------------------------------------------------

		/**
		*	Initialisation
		*/
		private:


        protected:
        void buildAnglesMap();



		public:
        void captureMouse(int msx, int msy);

		/**
		*	Add a move
		*/
		public:
		void addMove(int dx,int dy);

		/**
		*	Start the capture phase
		*/
		public:
        void startCapture(int mx, int my);

		/**
		*	Stop the capture phase
		*/
		public:
		int stopCapture(){
			return matchGesture();
		}

		/**
		*	Match the gesture
		*/
		public:
        int matchGesture();

		/**
		*	dif angle
		*/
		protected:
		unsigned int difAngle(int a,int b);

		/**
		*	return a filled 2D table
		*/
		protected:
        void fill2DTable(unsigned int w, unsigned int h, unsigned int f);

		/**
		*	cost Levenshtein
		*/
		protected:
        unsigned int costLeven(const vector<float>& a, const vector<float>& b);
	};



#endif /* GESTURES_HPP */

