#ifndef MAINPART_HPP
#define MAINPART_HPP

#include <vector>
#include <set>
#include <stack>
#include "boost/multi_array.hpp"
using namespace std;
using namespace boost;


#ifdef __APPLE__
#include <SDL/SDL.h>
#else
#include <SDL.h>
#endif


#include "Engine/engine.hpp"
#include "globals.hpp"
#include "entities.hpp"
#include "widget.hpp"
#include "gestures.hpp"



const int PLAY_WIDTH=640;
const int PLAY_HEIGHT=364;

enum MainState {ms_play,ms_dialog};


enum TriggerType {t_auto,t_tile, t_gesture};

struct Condition {
  TriggerType triggertype;
  int trigger;
};

//if from is -1, then ignore
struct Replacement {
    int from;
    int to;
};

struct EventDat {
    string s;
    Condition c;
    Replacement r;
};

struct Warning {
    string first;
    string subsequent;
    bool triggered;
};

class MainPart: public EnginePart
{
public:
    void reset();
    MainPart(SubengineInterface* subengines, Global_Variables* p_global_variables);
    ~MainPart();
protected:
    Global_Variables *global_variables;
    int input(const SDL_Event& event);
    int logic();
    void draw();
private:
    int map[8][10];
    void calcpos();
    int checkdir(int d);
    void movedir(int d);
    void charmove(int d);
    int px;
    int py;
    MainState mainstate;
    string s;
    vector<EventDat> events;
    vector<Warning> warnings;
    string crazy;
    int curevent;
    bool speaking;
    bool warning;
    bool havetalked;
    bool endlevel;
    bool adjacent(int t);
    MouseGesture* mousegesture;
    bool m1down;
    int curgesture;
    int gesturetimer;
    int score;
    void swapadjacent(int from, int to);
    void loadgestures();
    bool existsinmap(int t);
};



#endif /* MAINPART_HPP */
